@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "http/parser"
 
 require "http/errors"
@@ -20,6 +21,6 @@ module HTTP
 
   class << self
     # HTTP[:accept => 'text/html'].get(...)
-    alias_method :[], :headers
+    alias [] headers
   end
 end
