@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "base64"
 
 require "http/headers"
@@ -74,10 +75,12 @@ module HTTP
       branch(options).request verb, uri
     end
 
-    # @overload(options = {})
+    # @overload timeout(options = {})
     #   Syntax sugar for `timeout(:per_operation, options)`
-    # @overload(klass, options = {})
+    # @overload timeout(klass, options = {})
+    #   Adds a timeout to the request.
     #   @param [#to_sym] klass
+    #     either :null, :global, or :per_operation
     #   @param [Hash] options
     #   @option options [Float] :read Read timeout
     #   @option options [Float] :write Write timeout
@@ -92,7 +95,7 @@ module HTTP
               when :null          then HTTP::Timeout::Null
               when :global        then HTTP::Timeout::Global
               when :per_operation then HTTP::Timeout::PerOperation
-              else fail ArgumentError, "Unsupported Timeout class: #{klass}"
+              else raise ArgumentError, "Unsupported Timeout class: #{klass}"
               end
 
       [:read, :write, :connect].each do |k|
@@ -106,12 +109,13 @@ module HTTP
       )
     end
 
-    # @overload persistent(host)
+    # @overload persistent(host, timeout: 5)
     #   Flags as persistent
-    #   @param [String] host
-    #   @raise [Request::Error] if Host is invalid
+    #   @param  [String] host
+    #   @option [Integer] timeout Keep alive timeout
+    #   @raise  [Request::Error] if Host is invalid
     #   @return [HTTP::Client] Persistent client
-    # @overload persistent(host, &block)
+    # @overload persistent(host, timeout: 5, &block)
     #   Executes given block with persistent client and automatically closes
     #   connection at the end of execution.
     #
@@ -135,12 +139,13 @@ module HTTP
     #
     #   @yieldparam [HTTP::Client] client Persistent client
     #   @return [Object] result of last expression in the block
-    def persistent(host)
-      p_client = branch default_options.with_persistent host
+    def persistent(host, timeout: 5)
+      options  = {:keep_alive_timeout => timeout}
+      p_client = branch default_options.merge(options).with_persistent host
       return p_client unless block_given?
       yield p_client
     ensure
-      p_client.close
+      p_client.close if p_client
     end
 
     # Make a request through an HTTP proxy
@@ -152,14 +157,14 @@ module HTTP
       proxy_hash[:proxy_port]     = proxy[1] if proxy[1].is_a?(Integer)
       proxy_hash[:proxy_username] = proxy[2] if proxy[2].is_a?(String)
       proxy_hash[:proxy_password] = proxy[3] if proxy[3].is_a?(String)
+      proxy_hash[:proxy_headers]  = proxy[2] if proxy[2].is_a?(Hash)
+      proxy_hash[:proxy_headers]  = proxy[4] if proxy[4].is_a?(Hash)
 
-      if [2, 4].include?(proxy_hash.keys.size)
-        branch default_options.with_proxy(proxy_hash)
-      else
-        fail(RequestError, "invalid HTTP proxy: #{proxy_hash}")
-      end
+      raise(RequestError, "invalid HTTP proxy: #{proxy_hash}") unless (2..5).cover?(proxy_hash.keys.size)
+
+      branch default_options.with_proxy(proxy_hash)
     end
-    alias_method :through, :via
+    alias through via
 
     # Make client follow redirects.
     # @param opts
@@ -206,7 +211,7 @@ module HTTP
       user = opts.fetch :user
       pass = opts.fetch :pass
 
-      auth("Basic " << Base64.strict_encode64("#{user}:#{pass}"))
+      auth("Basic " + Base64.strict_encode64("#{user}:#{pass}"))
     end
 
     # Get options for HTTP
@@ -227,6 +232,14 @@ module HTTP
       branch default_options.with_nodelay(true)
     end
 
+    # Turn on given features. Available features are:
+    # * auto_inflate
+    # * auto_deflate
+    # @param features
+    def use(*features)
+      branch default_options.with_features(features)
+    end
+
     private
 
     # :nodoc:
