@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "forwardable"
 
 require "http/form_data"
@@ -13,10 +14,7 @@ module HTTP
     extend Forwardable
     include Chainable
 
-    KEEP_ALIVE         = "Keep-Alive".freeze
-    CLOSE              = "close".freeze
-
-    HTTP_OR_HTTPS_RE   = %r{^https?://}i
+    HTTP_OR_HTTPS_RE = %r{^https?://}i
 
     def initialize(default_options = {})
       @default_options = HTTP::Options.new(default_options)
@@ -67,12 +65,14 @@ module HTTP
       end
 
       res = Response.new(
-        :status     => @connection.status_code,
-        :version    => @connection.http_version,
-        :headers    => @connection.headers,
-        :connection => @connection,
-        :encoding   => options.encoding,
-        :uri        => req.uri
+        :status        => @connection.status_code,
+        :version       => @connection.http_version,
+        :headers       => @connection.headers,
+        :proxy_headers => @connection.proxy_response_headers,
+        :connection    => @connection,
+        :encoding      => options.encoding,
+        :auto_inflate  => options.feature(:auto_inflate),
+        :uri           => req.uri
       )
 
       @connection.finish_response if req.verb == :head
@@ -95,7 +95,7 @@ module HTTP
     # Verify our request isn't going to be made against another URI
     def verify_connection!(uri)
       if default_options.persistent? && uri.origin != default_options.persistent
-        fail StateError, "Persistence is enabled for #{default_options.persistent}, but we got #{uri.origin}"
+        raise StateError, "Persistence is enabled for #{default_options.persistent}, but we got #{uri.origin}"
       # We re-create the connection object because we want to let prior requests
       # lazily load the body as long as possible, and this mimics prior functionality.
       elsif @connection && (!@connection.keep_alive? || @connection.expired?)
@@ -137,13 +137,10 @@ module HTTP
       headers = opts.headers
 
       # Tell the server to keep the conn open
-      if default_options.persistent?
-        headers[Headers::CONNECTION] = KEEP_ALIVE
-      else
-        headers[Headers::CONNECTION] = CLOSE
-      end
+      headers[Headers::CONNECTION] = default_options.persistent? ? Connection::KEEP_ALIVE : Connection::CLOSE
 
       cookies = opts.cookies.values
+
       unless cookies.empty?
         cookies = opts.headers.get(Headers::COOKIE).concat(cookies).join("; ")
         headers[Headers::COOKIE] = cookies
@@ -154,17 +151,24 @@ module HTTP
 
     # Create the request body object to send
     def make_request_body(opts, headers)
-      case
-      when opts.body
-        opts.body
-      when opts.form
-        form = HTTP::FormData.create opts.form
-        headers[Headers::CONTENT_TYPE]   ||= form.content_type
-        headers[Headers::CONTENT_LENGTH] ||= form.content_length
-        form.to_s
-      when opts.json
-        headers[Headers::CONTENT_TYPE] ||= "application/json"
-        MimeType[:json].encode opts.json
+      request_body =
+        case
+        when opts.body
+          opts.body
+        when opts.form
+          form = HTTP::FormData.create opts.form
+          headers[Headers::CONTENT_TYPE]   ||= form.content_type
+          headers[Headers::CONTENT_LENGTH] ||= form.content_length
+          form.to_s
+        when opts.json
+          body = MimeType[:json].encode opts.json
+          headers[Headers::CONTENT_TYPE] ||= "application/json; charset=#{body.encoding.name}"
+          body
+        end
+      if (auto_deflate = opts.feature(:auto_deflate))
+        auto_deflate.deflate(headers, request_body)
+      else
+        request_body
       end
     end
   end
