@@ -1,5 +1,6 @@
-require 'json'
-require 'http/mime_type/adapter'
+# frozen_string_literal: true
+require "json"
+require "http/mime_type/adapter"
 
 module HTTP
   module MimeType
@@ -17,7 +18,7 @@ module HTTP
       end
     end
 
-    register_adapter 'application/json', JSON
-    register_alias   'application/json', :json
+    register_adapter "application/json", JSON
+    register_alias   "application/json", :json
   end
 end
