@@ -1,16 +1,26 @@
+# frozen_string_literal: true
 require "http/headers"
 require "openssl"
 require "socket"
 require "http/uri"
+require "http/feature"
+require "http/features/auto_inflate"
+require "http/features/auto_deflate"
 
 module HTTP
+  # rubocop:disable Metrics/ClassLength
   class Options
     @default_socket_class     = TCPSocket
     @default_ssl_socket_class = OpenSSL::SSL::SSLSocket
     @default_timeout_class    = HTTP::Timeout::Null
+    @available_features       = {
+      :auto_inflate => Features::AutoInflate,
+      :auto_deflate => Features::AutoDeflate
+    }
 
     class << self
       attr_accessor :default_socket_class, :default_ssl_socket_class, :default_timeout_class
+      attr_reader :available_features
 
       def new(options = {})
         return options if options.is_a?(self)
@@ -49,7 +59,8 @@ module HTTP
         :keep_alive_timeout => 5,
         :headers            => {},
         :cookies            => {},
-        :encoding           => nil
+        :encoding           => nil,
+        :features           => {}
       }
 
       opts_w_defaults = defaults.merge(options)
@@ -72,6 +83,38 @@ module HTTP
       self.encoding = Encoding.find(encoding)
     end
 
+    def_option :features do |features|
+      # Normalize features from:
+      #
+      #     [{feature_one: {opt: 'val'}}, :feature_two]
+      #
+      # into:
+      #
+      #     {feature_one: {opt: 'val'}, feature_two: {}}
+      features = features.each_with_object({}) do |feature, h|
+        if feature.is_a?(Hash)
+          h.merge!(feature)
+        else
+          h[feature] = {}
+        end
+      end
+
+      self.features.merge(features)
+    end
+
+    def features=(features)
+      @features = features.each_with_object({}) do |(name, opts_or_feature), h|
+        h[name] = if opts_or_feature.is_a?(Feature)
+                    opts_or_feature
+                  else
+                    unless (feature = self.class.available_features[name])
+                      argument_error! "Unsupported feature: #{name}"
+                    end
+                    feature.new(opts_or_feature)
+                  end
+      end
+    end
+
     %w(
       proxy params form json body follow response
       socket_class nodelay ssl_socket_class ssl_context ssl
@@ -126,6 +169,10 @@ module HTTP
       dupped
     end
 
+    def feature(name)
+      features[name]
+    end
+
     protected
 
     def []=(option, val)
@@ -135,7 +182,7 @@ module HTTP
     private
 
     def argument_error!(message)
-      fail(Error, message, caller[1..-1])
+      raise(Error, message, caller[1..-1])
     end
   end
 end
