@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "forwardable"
 require "base64"
 require "time"
@@ -23,26 +24,31 @@ module HTTP
     # Default User-Agent header value
     USER_AGENT = "http.rb/#{HTTP::VERSION}".freeze
 
-    # RFC 2616: Hypertext Transfer Protocol -- HTTP/1.1
-    METHODS = [:options, :get, :head, :post, :put, :delete, :trace, :connect]
+    METHODS = [
+      # RFC 2616: Hypertext Transfer Protocol -- HTTP/1.1
+      :options, :get, :head, :post, :put, :delete, :trace, :connect,
 
-    # RFC 2518: HTTP Extensions for Distributed Authoring -- WEBDAV
-    METHODS.concat [:propfind, :proppatch, :mkcol, :copy, :move, :lock, :unlock]
+      # RFC 2518: HTTP Extensions for Distributed Authoring -- WEBDAV
+      :propfind, :proppatch, :mkcol, :copy, :move, :lock, :unlock,
 
-    # RFC 3648: WebDAV Ordered Collections Protocol
-    METHODS.concat [:orderpatch]
+      # RFC 3648: WebDAV Ordered Collections Protocol
+      :orderpatch,
 
-    # RFC 3744: WebDAV Access Control Protocol
-    METHODS.concat [:acl]
+      # RFC 3744: WebDAV Access Control Protocol
+      :acl,
 
-    # draft-dusseault-http-patch: PATCH Method for HTTP
-    METHODS.concat [:patch]
+      # RFC 6352: vCard Extensions to WebDAV -- CardDAV
+      :report,
 
-    # draft-reschke-webdav-search: WebDAV Search
-    METHODS.concat [:search]
+      # RFC 5789: PATCH Method for HTTP
+      :patch,
+
+      # draft-reschke-webdav-search: WebDAV Search
+      :search
+    ].freeze
 
     # Allowed schemes
-    SCHEMES = [:http, :https, :ws, :wss]
+    SCHEMES = [:http, :https, :ws, :wss].freeze
 
     # Default ports of supported schemes
     PORTS = {
@@ -50,7 +56,7 @@ module HTTP
       :https  => 443,
       :ws     => 80,
       :wss    => 443
-    }
+    }.freeze
 
     # Method is given as a lowercase symbol e.g. :get, :post
     attr_reader :verb
@@ -71,11 +77,11 @@ module HTTP
     # @option opts [String] :body
     def initialize(opts)
       @verb   = opts.fetch(:verb).to_s.downcase.to_sym
-      @uri    = normalize_uri(opts.fetch :uri)
+      @uri    = normalize_uri(opts.fetch(:uri))
       @scheme = @uri.scheme.to_s.downcase.to_sym if @uri.scheme
 
-      fail(UnsupportedMethodError, "unknown method: #{verb}") unless METHODS.include?(@verb)
-      fail(UnsupportedSchemeError, "unknown scheme: #{scheme}") unless SCHEMES.include?(@scheme)
+      raise(UnsupportedMethodError, "unknown method: #{verb}") unless METHODS.include?(@verb)
+      raise(UnsupportedSchemeError, "unknown scheme: #{scheme}") unless SCHEMES.include?(@scheme)
 
       @proxy   = opts[:proxy] || {}
       @body    = opts[:body]
@@ -103,7 +109,7 @@ module HTTP
 
     # Stream the request to a socket
     def stream(socket)
-      include_proxy_authorization_header if using_authenticated_proxy? && !@uri.https?
+      include_proxy_headers if using_proxy? && !@uri.https?
       Request::Writer.new(socket, body, headers, headline).stream
     end
 
@@ -114,7 +120,12 @@ module HTTP
 
     # Is this request using an authenticated proxy?
     def using_authenticated_proxy?
-      proxy && proxy.keys.size == 4
+      proxy && proxy.keys.size >= 4
+    end
+
+    def include_proxy_headers
+      headers.merge!(proxy[:proxy_headers]) if proxy.key?(:proxy_headers)
+      include_proxy_authorization_header if using_authenticated_proxy?
     end
 
     # Compute and add the Proxy-Authorization header
@@ -134,7 +145,7 @@ module HTTP
 
     # Compute HTTP request header for direct or proxy request
     def headline
-      request_uri = using_proxy? ? uri : uri.omit(:scheme, :authority)
+      request_uri = (using_proxy? && !uri.https?) ? uri : uri.omit(:scheme, :authority)
       "#{verb.to_s.upcase} #{request_uri.omit :fragment} HTTP/#{version}"
     end
 
@@ -151,7 +162,7 @@ module HTTP
       )
 
       connect_headers[Headers::PROXY_AUTHORIZATION] = proxy_authorization_header if using_authenticated_proxy?
-
+      connect_headers.merge!(proxy[:proxy_headers]) if proxy.key?(:proxy_headers)
       connect_headers
     end
 
