@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "http/headers"
 
 module HTTP
@@ -16,7 +17,7 @@ module HTTP
       CHUNKED_END = "#{ZERO}#{CRLF}#{CRLF}".freeze
 
       # Types valid to be used as body source
-      VALID_BODY_TYPES = [String, NilClass, Enumerable]
+      VALID_BODY_TYPES = [String, NilClass, Enumerable].freeze
 
       def initialize(socket, body, headers, headline)
         @body           = body
@@ -52,8 +53,10 @@ module HTTP
       def add_body_type_headers
         if @body.is_a?(String) && !@headers[Headers::CONTENT_LENGTH]
           @request_header << "#{Headers::CONTENT_LENGTH}: #{@body.bytesize}"
+        elsif @body.nil? && !@headers[Headers::CONTENT_LENGTH]
+          @request_header << "#{Headers::CONTENT_LENGTH}: 0"
         elsif @body.is_a?(Enumerable) && CHUNKED != @headers[Headers::TRANSFER_ENCODING]
-          fail(RequestError, "invalid transfer encoding")
+          raise(RequestError, "invalid transfer encoding")
         end
       end
 
@@ -62,7 +65,7 @@ module HTTP
       def join_headers
         # join the headers array with crlfs, stick two on the end because
         # that ends the request header
-        @request_header.join(CRLF) + (CRLF) * 2
+        @request_header.join(CRLF) + CRLF * 2
       end
 
       def send_request
@@ -85,7 +88,7 @@ module HTTP
           end
 
           write(CHUNKED_END)
-        else fail TypeError, "invalid body type: #{@body.class}"
+        else raise TypeError, "invalid body type: #{@body.class}"
         end
       end
 
@@ -94,17 +97,14 @@ module HTTP
       def write(data)
         until data.empty?
           length = @socket.write(data)
-          if data.length > length
-            data = data[length..-1]
-          else
-            break
-          end
+          break unless data.bytesize > length
+          data = data.byteslice(length..-1)
         end
       end
 
       def validate_body_type!
         return if VALID_BODY_TYPES.any? { |type| @body.is_a? type }
-        fail RequestError, "body of wrong type: #{@body.class}"
+        raise RequestError, "body of wrong type: #{@body.class}"
       end
     end
   end
