@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "forwardable"
 require "http/client"
 
@@ -9,17 +10,23 @@ module HTTP
       include Enumerable
       def_delegator :to_s, :empty?
 
-      def initialize(client, encoding = Encoding::BINARY)
-        @client    = client
-        @streaming = nil
-        @contents  = nil
-        @encoding  = encoding
+      # The connection object used to make the corresponding request.
+      #
+      # @return [HTTP::Connection]
+      attr_reader :connection
+
+      def initialize(stream, encoding = Encoding::BINARY)
+        @stream     = stream
+        @connection = stream.is_a?(Inflater) ? stream.connection : stream
+        @streaming  = nil
+        @contents   = nil
+        @encoding   = encoding
       end
 
       # (see HTTP::Client#readpartial)
       def readpartial(*args)
         stream!
-        @client.readpartial(*args)
+        @stream.readpartial(*args)
       end
 
       # Iterate over the body, allowing it to be enumerable
@@ -33,13 +40,21 @@ module HTTP
       def to_s
         return @contents if @contents
 
-        fail StateError, "body is being streamed" unless @streaming.nil?
+        raise StateError, "body is being streamed" unless @streaming.nil?
+
+        # see issue 312
+        begin
+          encoding = Encoding.find @encoding
+        rescue ArgumentError
+          encoding = Encoding::BINARY
+        end
 
         begin
-          @streaming = false
-          @contents = "".force_encoding(@encoding)
-          while (chunk = @client.readpartial)
-            @contents << chunk.force_encoding(@encoding)
+          @streaming  = false
+          @contents   = String.new("").force_encoding(encoding)
+
+          while (chunk = @stream.readpartial)
+            @contents << chunk.force_encoding(encoding)
           end
         rescue
           @contents = nil
@@ -48,11 +63,11 @@ module HTTP
 
         @contents
       end
-      alias_method :to_str, :to_s
+      alias to_str to_s
 
       # Assert that the body is actively being streamed
       def stream!
-        fail StateError, "body has already been consumed" if @streaming == false
+        raise StateError, "body has already been consumed" if @streaming == false
         @streaming = true
       end
 
