@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module HTTP
   class Response
     class Parser
@@ -11,7 +12,7 @@ module HTTP
       def add(data)
         @parser << data
       end
-      alias_method :<<, :add
+      alias << add
 
       def headers?
         !!@headers
