@@ -1,7 +1,8 @@
+# frozen_string_literal: true
 module ServerRunner
-  def run_server(name, &block)
+  def run_server(name)
     let! name do
-      server = block.call
+      server = yield
 
       Thread.new { server.start }
 
